from bcf_api_xml.models import XYZ
from lxml import builder


def to_xml(plane):
    e = builder.ElementMaker()
    return e.ClippingPlane(
        e.Location(*XYZ.to_xml(plane["location"])),
        e.Direction(*XYZ.to_xml(plane["direction"])),
    )


def to_python(xml):
    return {
        "location": XYZ.to_python(xml.find("Location")),
        "direction": XYZ.to_python(xml.find("Direction")),
    }



from bcf_api_xml.models import Component
from lxml import builder


def boolean_repr(value):
    return "true" if value else "false"


def to_boolean(value):
    return value == "true"


def to_xml(visibility):
    e = builder.ElementMaker()
    children = []
    if exceptions := visibility.get("exceptions"):
        components = [
            Component.to_xml(component)
            for component in exceptions
            if component.get("ifc_guid")
        ]
        children.append(e.Exceptions(*components))
    return e.Visibility(
        *children, DefaultVisibility=boolean_repr(visibility["default_visibility"])
    )


def to_python(xml):
    visibility = {
        "default_visibility": xml.get("DefaultVisibility"),
    }
    if (exceptions := xml.find("Exceptions")) is not None:
        visibility["exceptions"] = [
            Component.to_python(component) for component in exceptions.findall("Component")
        ]

    return visibility



from bcf_api_xml.models import XYZ
from lxml import builder


def to_xml(line):
    e = builder.ElementMaker()
    return e.Line(
        e.StartPoint(*XYZ.to_xml(line["start_point"])),
        e.EndPoint(*XYZ.to_xml(line["end_point"])),
    )


def to_python(xml):
    return {
        "start_point": XYZ.to_python(xml.find("StartPoint")),
        "end_point": XYZ.to_python(xml.find("EndPoint")),
    }



from lxml import builder


def to_xml(component):
    e = builder.ElementMaker()
    children = []
    if (originating_system := component.get("originating_system")) is not None:
        children.append(e.OriginatingSystem(originating_system))
    if (authoring_tool_id := component.get("authoring_tool_id")) is not None:
        children.append(e.AuthoringToolId(authoring_tool_id))

    return e.Component(*children, IfcGuid=component["ifc_guid"])


def to_python(xml):
    component = {"ifc_guid": xml.get("IfcGuid")}
    if (originating_system := xml.find("OriginatingSystem")) is not None:
        component["originating_system"] = originating_system.text

    if (authoring_tool_id := xml.find("AuthoringToolId")) is not None:
        component["authoring_tool_id"] = authoring_tool_id.text
    return component



from bcf_api_xml.models import XYZ
from lxml import builder


def to_xml(camera):
    e = builder.ElementMaker()
    return e.OrthogonalCamera(
        e.CameraViewPoint(*XYZ.to_xml(camera["camera_view_point"])),
        e.CameraDirection(*XYZ.to_xml(camera["camera_direction"])),
        e.CameraUpVector(*XYZ.to_xml(camera["camera_up_vector"])),
        e.ViewToWorldScale(str(camera["view_to_world_scale"])),
    )


def to_python(xml):
    return {
        "camera_view_point": XYZ.to_python(xml.find("CameraViewPoint")),
        "camera_direction": XYZ.to_python(xml.find("CameraDirection")),
        "camera_up_vector": XYZ.to_python(xml.find("CameraUpVector")),
        "view_to_world_scale": float(xml.find("ViewToWorldScale").text),
    }



from lxml import builder


def to_xml(point):
    e = builder.ElementMaker()
    return (e.X(str(point["x"])), e.Y(str(point["y"])), e.Z(str(point["z"])))


def to_python(xml):
    return {
        "x": float(xml.find("X").text),
        "y": float(xml.find("Y").text),
        "z": float(xml.find("Z").text),
    }



from lxml import builder
from bcf_api_xml.models import (
    Component,
    OrthogonalCamera,
    PerspectiveCamera,
    Line,
    ClippingPlane,
    ViewSetupHints,
    Visibility,
    Color,
)


def to_xml(viewpoint):
    e = builder.ElementMaker()

    children = []

    if (components := viewpoint.get("components")) is not None:
        visibility = components.get("visibility")

        components_children = []

        if visibility and (view_setup_hints := visibility.get("view_setup_hints")) is not None:
            components_children.append(ViewSetupHints.to_xml(view_setup_hints))

        xml_selections = [
            Component.to_xml(component)
            for component in components.get("selection", [])
            if component.get("ifc_guid")
        ]
        if xml_selections:
            components_children.append(e.Selection(*xml_selections))

        if visibility:
            components_children.append(Visibility.to_xml(visibility))

        xml_colorings = [Color.to_xml(coloring) for coloring in components.get("coloring", [])]
        if xml_colorings:
            components_children.append(e.Coloring(*xml_colorings))

        children.append(e.Components(*components_children))

    if (orthogonal_camera := viewpoint.get("orthogonal_camera")) is not None:
        xml_ortogonal_camera = OrthogonalCamera.to_xml(orthogonal_camera)
        children.append(xml_ortogonal_camera)

    if (perspective_camera := viewpoint.get("perspective_camera")) is not None:
        xml_perspective_camera = PerspectiveCamera.to_xml(perspective_camera)
        children.append(xml_perspective_camera)

    xml_lines = [Line.to_xml(line) for line in viewpoint.get("lines", [])]
    if xml_lines:
        children.append(e.Lines(*xml_lines))

    xml_planes = [
        ClippingPlane.to_xml(plane) for plane in viewpoint.get("clipping_planes", [])
    ]
    if xml_planes:
        children.append(e.ClippingPlanes(*xml_planes))

    return e.VisualizationInfo(*children, Guid=str(viewpoint["guid"]))


def to_python(xml):
    viewpoint = {}
    if (perspective_camera := xml.find("PerspectiveCamera")) is not None:
        viewpoint["perspective_camera"] = PerspectiveCamera.to_python(perspective_camera)
    if (orthogonal_camera := xml.find("OrthogonalCamera")) is not None:
        viewpoint["orthogonal_camera"] = OrthogonalCamera.to_python(orthogonal_camera)

    if (lines := xml.find("Lines")) is not None:
        viewpoint["lines"] = [
            Line.to_python(line)
            for line in lines.findall("Line")
        ]
    if (planes := xml.find("ClippingPlanes")) is not None:
        viewpoint["clipping_planes"] = [
            ClippingPlane.to_python(plane)
            for plane in planes.findall("ClippingPlane")
        ]

    if (components := xml.find("Components")) is not None:
        viewpoint["components"] = {}
        if (selection := components.find("Selection")) is not None:
            viewpoint["components"]["selection"] = [
                Component.to_python(component) for component in selection.findall("Component")
            ]
        if (visibility := components.find("Visibility")) is not None:
            viewpoint["components"]["visibility"] = Visibility.to_python(visibility)
            if (hints := components.find("ViewSetupHints")) is not None:
                viewpoint["components"]["visibility"][
                    "view_setup_hints"
                ] = ViewSetupHints.to_python(hints)

        if (colors := components.find("Coloring")) is not None:
            viewpoint["components"]["coloring"] = [
                Color.to_python(color) for color in colors.findall("Color")
            ]

    return viewpoint



import io
import base64
import zipfile
from os import path
from lxml import etree, builder
from bcf_api_xml.models import Topic, Comment, VisualizationInfo, Viewpoint
from bcf_api_xml.errors import InvalidBCF

SCHEMA_DIR = path.realpath(path.join(path.dirname(__file__), "Schemas"))


def is_valid(schema_name, xml, raise_exception=False):
    schema_path = path.join(SCHEMA_DIR, schema_name)
    with open(schema_path, "r") as file:
        schema = etree.XMLSchema(file=file)

    if not schema.validate(xml):
        if raise_exception:
            raise InvalidBCF(schema.error_log)
        else:
            print(schema.error_log)
        return False
    return True


def export_markup(topic, comments, viewpoints):
    e = builder.ElementMaker()
    children = [Topic.to_xml(topic)]

    for comment in comments:
        children.append(Comment.to_xml(comment))

    for index, viewpoint in enumerate(viewpoints):
        xml_viewpoint = Viewpoint.to_xml(viewpoint, index == 0)
        children.append(xml_viewpoint)
    xml_markup = e.Markup(*children)
    is_valid("markup.xsd", xml_markup, raise_exception=True)
    return xml_markup


def write_xml(zf, path, xml):
    data = etree.tostring(xml, encoding="utf-8", pretty_print=True, xml_declaration=True)
    zf.writestr(path, data)


def to_zip(topics, comments, viewpoints):
    """
    topics: list of topics (dict parsed from BCF-API json)
    viewpoints: dict(topics_guid=[viewpoint])
    comments: dict(topics_guid=[comment])
    """
    zip_file = io.BytesIO()
    with zipfile.ZipFile(zip_file, mode="w", compression=zipfile.ZIP_DEFLATED) as zf:
        with open(path.join(SCHEMA_DIR, "bcf.version"), "rb") as version_file:
            zf.writestr("bcf.version", version_file.read())

        for topic in topics:
            topic_guid = topic["guid"]
            topic_comments = comments.get(topic_guid, [])
            topic_viewpoints = viewpoints.get(topic_guid, [])
            # 1 directory per topic
            topic_dir = topic_guid + "/"
            zfi = zipfile.ZipInfo(topic_dir)
            zf.writestr(zfi, "")  # create the directory in the zip

            xml_markup = export_markup(topic, topic_comments, topic_viewpoints)
            write_xml(zf, topic_dir + "markup.bcf", xml_markup)

            for index, viewpoint in enumerate(topic_viewpoints):
                xml_visinfo = VisualizationInfo.to_xml(viewpoint)
                viewpoint_name = (
                    "viewpoint.bcfv" if index == 0 else (viewpoint["guid"] + ".bcfv")
                )
                write_xml(zf, topic_dir + viewpoint_name, xml_visinfo)
                # snapshots
                if viewpoint.get("snapshot"):
                    snapshot_name = (
                        "snapshot.png" if index == 0 else (viewpoint["guid"] + ".png")
                    )
                    snapshot = viewpoint.get("snapshot").get("snapshot_data")
                    # Break out the header from the base64 content
                    _, data = snapshot.split(";base64,")
                    zf.writestr(topic_dir + snapshot_name, base64.b64decode(data))
    return zip_file



from bcf_api_xml.models import Component
from lxml import builder


def to_xml(coloring):
    e = builder.ElementMaker()
    return e.Color(
        *[Component.to_xml(component) for component in coloring["components"]],
        Color=coloring["color"],
    )


def to_python(xml):
    return {
        "color": xml.get("Color"),
        "components": [
            Component.to_python(component) for component in xml.findall("Component")
        ],
    }



from lxml import builder


def boolean_repr(value):
    return "true" if value else "false"


def to_boolean(value):
    return value == "true"


def to_xml(hints):
    e = builder.ElementMaker()
    return e.ViewSetupHints(
        SpacesVisible=boolean_repr(hints["spaces_visible"]),
        SpaceBoundariesVisible=boolean_repr(hints["space_boundaries_visible"]),
        OpeningsVisible=boolean_repr(hints["openings_visible"]),
    )


def to_python(xml):
    return {
        "spaces_visible": to_boolean(xml.get("SpacesVisible")),
        "space_boundaries_visible": to_boolean(xml.get("SpaceBoundariesVisible")),
        "openings_visible": to_boolean(xml.get("OpeningsVisible")),
    }



from dateutil.parser import parse

from lxml import builder


def to_xml(comment):
    e = builder.ElementMaker()
    children = [
        e.Date(comment["date"]),
        e.Author(comment.get("author", "")),
        e.Comment(comment["comment"]),
    ]
    if (viewpoint_guid := comment.get("viewpoint_guid")) is not None:
        children.append(e.Viewpoint(Guid=str(viewpoint_guid)))
    if (modified_date := comment.get("modified_date")) is not None:
        children.append(e.ModifiedDate(modified_date))
    if (modified_author := comment.get("modified_author")) is not None:
        children.append(e.ModifiedAuthor(modified_author))

    return e.Comment(*children, Guid=str(comment["guid"]))


def to_python(xml):
    comment = {}

    comment["date"] = parse(xml.find("Date").text)
    comment["comment"] = xml.find("Comment").text or ""
    comment["author"] = xml.find("Author").text

    if (viewpoint := xml.find("Viewpoint")) is not None:
        comment["viewpoint_guid"] = viewpoint.get("Guid")

    if (modified_date := xml.find("ModifiedDate")) is not None:
        comment["modified_date"] = parse(modified_date.text)

    if (modified_author := xml.find("ModifiedAuthor")) is not None:
        comment["modified_author"] = modified_author.text

    return comment



from lxml import builder


def to_xml(viewpoint, is_first):
    e = builder.ElementMaker()
    viewpoint_name = "viewpoint.bcfv" if is_first else (viewpoint["guid"] + ".bcfv")
    snapshot_name = "snapshot.png" if is_first else (viewpoint["guid"] + ".png")

    return e.Viewpoints(
        e.Viewpoint(viewpoint_name),
        e.Snapshot(snapshot_name),
        e.Index(str(viewpoint["index"])),
        Guid=str(viewpoint["guid"]),
    )


def to_python(xml):
    viewpoint = {"guid": xml.get("Guid")}

    if (index := xml.find("Index")) is not None:
        viewpoint["index"] = int(index.text)

    if (snapshot_filename := xml.find("Snapshot")) is not None:
        viewpoint["snapshot_filename"] = snapshot_filename.text

    if (viewpoint_filename := xml.find("Viewpoint")) is not None:
        viewpoint["viewpoint_filename"] = viewpoint_filename.text

    return viewpoint



import io
import os
import base64
import zipfile
from lxml import etree
from .errors import UnsupportedBCFVersion
from bcf_api_xml.models import (
    Topic,
    Comment,
    Viewpoint,
    VisualizationInfo,
)


def check_bcf_version(file):
    bcf_version_tree = etree.parse(file)
    root = bcf_version_tree.getroot()
    version = root.get("VersionId")
    if version != "2.1":
        raise UnsupportedBCFVersion(
            f"version {version} is not supported. Only BCF 2.1 is supported"
        )


def to_json(bcf_file):
    with zipfile.ZipFile(bcf_file, "r") as zip_ref:
        try:
            with zip_ref.open("bcf.version") as version_file:
                check_bcf_version(version_file)
        except KeyError:
            raise UnsupportedBCFVersion(
                f"Unable to check version. It's probably an oudated version. Only BCF 2.1 is supported"
            )
        files = zip_ref.infolist()
        all_topics = []
        for file in files:
            if file.is_dir():
                # if zip has explicit directories, ignore them
                continue
            if file.filename.endswith("markup.bcf"):
                markup = file.filename
                topic_directory = os.path.dirname(file.filename) + "/"

                root = etree.fromstring(zip_ref.read(markup))
                xml_topic = root.find("Topic")
                topic = Topic.to_python(xml_topic)
                topic["comments"] = [
                    Comment.to_python(comment_xml) for comment_xml in root.findall("Comment")
                ]
                viewpoints = [
                    Viewpoint.to_python(viewpoint_xml)
                    for viewpoint_xml in root.findall("Viewpoints")
                ]
                for viewpoint in viewpoints:
                    if filename := viewpoint.pop("snapshot_filename", None):
                        file = io.BytesIO(zip_ref.read(topic_directory + filename))
                        file.name = filename
                        viewpoint["snapshot"] = {
                            "snapshot_type": os.path.splitext(filename)[1],
                            "snapshot_data": file,
                        }
                    if filename := viewpoint.pop("viewpoint_filename", None):
                        xml = etree.fromstring(zip_ref.read(topic_directory + filename))
                        viewpoint.update(**VisualizationInfo.to_python(xml))
                topic["viewpoints"] = viewpoints
                all_topics.append(topic)
    return all_topics



from dateutil.parser import parse
from lxml import builder


def to_xml(topic):
    e = builder.ElementMaker()
    children = []
    for ref in topic.get("reference_links", []):
        children.append(e.ReferenceLink(ref))
    children.append(e.Title(topic["title"]))
    if (priority := topic.get("priority")) is not None:
        children.append(e.Priority(priority))
    if (index := topic.get("index")) is not None:
        children.append(e.Index(str(index)))
    for label in topic.get("labels"):
        children.append(e.Labels(label))

    children.append(e.CreationDate(topic["creation_date"]))
    children.append(e.CreationAuthor(topic.get("creation_author", "")))
    if (modified_date := topic.get("modified_date")) is not None:
        children.append(e.ModifiedDate(modified_date))
    if (modified_author := topic.get("modified_author")) is not None:
        children.append(e.ModifiedAuthor(modified_author))
    if (due_date := topic.get("due_date")) is not None:
        children.append(e.DueDate(due_date))
    if (assigned_to := topic.get("assigned_to")) is not None:
        children.append(e.AssignedTo(assigned_to))
    if (stage := topic.get("stage")) is not None:
        children.append(e.Stage(stage))
    if (description := topic.get("description")) is not None:
        children.append(e.Description(description))

    attributes = {"Guid": topic["guid"]}
    if (topic_type := topic.get("topic_type")) is not None:
        attributes["TopicType"] = topic_type

    if (topic_status := topic.get("topic_status")) is not None:
        attributes["TopicStatus"] = topic_status
    return e.Topic(*children, **attributes)


def to_python(xml):
    topic = {
        "guid": xml.get("Guid"),
        "topic_type": xml.get("TopicType"),
        "topic_status": xml.get("TopicStatus"),
        "title": xml.find("Title").text,
        "creation_date": parse(xml.find("CreationDate").text),
        "creation_author": xml.find("CreationAuthor").text,
    }

    if (priority := xml.find("Priority")) is not None:
        topic["priority"] = priority.text

    if (index := xml.find("Index")) is not None:
        topic["index"] = index.text

    if (creation_date := xml.find("CreationDate")) is not None:
        topic["creation_date"] = parse(creation_date.text)

    if (due_date := xml.find("DueDate")) is not None:
        topic["due_date"] = parse(due_date.text)

    if (creation_author := xml.find("CreationAuthor")) is not None:
        topic["creation_author"] = creation_author.text

    if (modified_date := xml.find("ModifiedDate")) is not None:
        topic["modified_date"] = parse(modified_date.text)

    if (modified_author := xml.find("ModifiedAuthor")) is not None:
        topic["modified_author"] = modified_author.text

    if (assigned_to := xml.find("AssignedTo")) is not None:
        topic["assigned_to"] = assigned_to.text

    if (description := xml.find("Description")) is not None:
        topic["description"] = description.text

    return topic



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['bcf_api_xml', 'bcf_api_xml.models']

package_data = \
{'': ['*'], 'bcf_api_xml': ['Schemas/*']}

install_requires = \
['generateDS>=2.35.13,<3.0.0',
 'lxml>=4.5.0,<5.0.0',
 'python-dateutil>=2.8.1,<3.0.0']

setup_kwargs = {
    'name': 'bcf-api-xml',
    'version': '0.2.10',
    'description': 'Convert BCF-API to BCF-XML',
    'long_description': None,
    'author': 'Hugo Duroux',
    'author_email': 'hugo@bimdata.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
